#
# [name] setup.py
#
# Written by Yoshikazu NAKAJIMA
#

from setuptools import setup, find_packages

setup(
	name='nkj',
	version='1.01',
	description='Surgical Navigation Kernel and Job Control Library',
	author='Yoshikazu NAKAJIMA',
	license='bmi.tmdu',
	packages=find_packages()
)
